local lg = love.graphics
local lp = love.physics

local leftWall = 20
local topWall = 20
local rightWall = 20
local bottomWall = 20
local wWidth = 800
local wHeight = 600
local ballSpeed = 600
local jointFreq = 6
local ballStartX = 400
local ballStartY = 330

local row1 = 150
local row2 = 280
local boxW = 14
local boxH = 60
local boxesPerRow = 14

local playfieldW = wWidth - leftWall - rightWall
local playfieldH = wHeight - topWall - bottomWall

-- Linear intERPolation
local function lerp(a, b, t)
  return t < 0.5 and a + (b-a) * t or b - (b-a) * (1-t)
end

local world = lp.newWorld(0, 0, false)

local function newRectElem(x, y, w, h, kind)
  local elem = {
    shape = lp.newRectangleShape(w, h);
    body = lp.newBody(world, x + w/2, y + h/2, kind);
  }
  elem.fixture = lp.newFixture(elem.body, elem.shape)
  return elem
end

local ball = {
  shape = lp.newCircleShape(10);
  body = lp.newBody(world, ballStartX, ballStartY, "dynamic");
  trail = {};
  vel = ballSpeed;
}
ball.fixture = lp.newFixture(ball.body, ball.shape)
ball.fixture:setRestitution(1) -- perfect bounces
ball.fixture:setFriction(0) -- prevents energy loss due to friction

local borders = {
  newRectElem(0, 0, leftWall, wHeight, "static"),
  newRectElem(0, 0, wWidth, topWall, "static"),
  newRectElem(wWidth - rightWall, 0, rightWall, wHeight, "static"),
  newRectElem(0, wHeight-bottomWall, wWidth, bottomWall, "static"),
}

local boxes = {}
local joints = {}

do
  local newJoint = 0
  local sepBetweenBoxes = (playfieldW+boxW)/(boxesPerRow+1)
  for row = 0, 1 do

    for i = 1, boxesPerRow do
      local box = newRectElem(i*sepBetweenBoxes - boxW + leftWall,
         lerp(row1, row2, row) - boxH/2, boxW, boxH, "dynamic")
      box.fixture:setRestitution(1) -- perfect bounces
      box.fixture:setFriction(0) -- no friction
      local boxidx = row*boxesPerRow + i
      boxes[boxidx] = box
      local prevbox
      if i == 1 then
        prevbox = borders[1]
      else
        prevbox = boxes[boxidx - 1]
      end
      newJoint = newJoint + 1
      joints[newJoint] = love.physics.newDistanceJoint(prevbox.body, box.body,
        (i-1)*sepBetweenBoxes + leftWall, lerp(row1, row2, row),
        i*sepBetweenBoxes + leftWall - boxW, lerp(row1, row2, row))
      joints[newJoint]:setFrequency(jointFreq) -- adjusts spring constant
    end
    newJoint = newJoint + 1
    joints[newJoint] = love.physics.newDistanceJoint(
      boxes[(row+1)*boxesPerRow].body, borders[3].body,
      (boxesPerRow)*sepBetweenBoxes + leftWall, lerp(row1, row2, row),
      (boxesPerRow+1)*sepBetweenBoxes + leftWall - boxW, lerp(row1, row2, row))
    joints[newJoint]:setFrequency(jointFreq) -- adjusts spring constant
  end
end

local function drawPoly(elem, filltype)
  lg.polygon(filltype, elem.body:getWorldPoints(elem.shape:getPoints()))
end

function love.keypressed(k)
  if k == "escape" then love.event.quit() end
end

function love.update(dt)
  world:update(dt)
  if #ball.trail > 10 then
    table.remove(ball.trail, 1)
    table.remove(ball.trail, 1)
  end
  table.insert(ball.trail, ball.body:getX())
  table.insert(ball.trail, ball.body:getY())
  do
    local x, y = ball.body:getLinearVelocity()
    x = x + math.random() - 0.5
    y = y + math.random() - 0.5
    local L = math.sqrt(x*x + y*y)
    if L < 1e-300 then
      x = 1
      y = 1
      L = 1.4142135623730951
    end

    -- Set the velocity to ball.vel with the same approx. direction as before
    ball.body:setLinearVelocity(x / L * ball.vel, y / L * ball.vel)
  end
end

function love.draw()
  lg.setColor(0.5, 0.5, 0.5)
  for i = 1, #borders do
    drawPoly(borders[i], "fill")
  end
  lg.setColor(1, 1, 1)
  
  for i = 1, #ball.trail, 2 do
    lg.circle("fill", ball.trail[i], ball.trail[i+1], lerp(2, 8, i/(#ball.trail-1)))
  end

  for i = 1, #boxes do
    lg.setColor(0.8, 0.7, 0.4)
    drawPoly(boxes[i], "line")
    local x, y = 0, 0
    local x1, y1, x2, y2 = boxes[i].body:getWorldPoints(x - boxW/2, y,
      x + boxW/2, y)
    lg.setColor(0.6, 0.6, 0.6)
    lg.line(x1, y1, x2, y2)
  end

  lg.setColor(0.6, 0.6, 0.6)
  for i = 1, #joints do
    lg.line(joints[i]:getAnchors())
  end
end
